/*============================================================
// Librairie RC5.h 
// Auteur : Philippe Mariano
// Cre le 10/07/06
// Mise  jour: 07/06
// Version 1.1
Cette librairie contient les sous-programmes 
ncessaires au dcodage et la dsrialisation 
d'une trame RC5 (voir description ci-dessous)
Composant cible: AT90S8535 
==============================================================*/

/*------------------Gnralits code RC5------------------------
Transmission d'un caractre toutes les 113,776ms
Dure d'un caractre: 24,889ms ( dduire du temps prcdent)
pour calculer le temps inter-caractre
Temps bit = 1,778ms (frquence = 36kHz)
Rapport cyclique: 1/2
Codage bit: Manchester 

    -----         ----
    |                 |
    |                 |
----                  ----
    1                 0
--------------------------------------------------------------*/

/*----------------------------------------------------------------------------------
Description du principe retenu pour le dcodage et la dsrialisation de la trame RC5

Matriel
On suppose que la rception du signal IR est ralis par un rcepteur type TSOP1836 
et que la sortie de ce composant est relie  l'entre ICP du microcontrleur.   

Logiciel
Le coeur du traitement est un graphe  quatre tats + un tat d'attente (cod par une 
structure switch ... case) plac au sein du sous-programme d'interruption <timer1_capt_isr>. 
Ce graphe est charg de retrouver les 14 bits de la trame RC5  partir du temps coul 
entre chaque front et de les dsrialiser dans une variable globale. <timer1_capt_isr>
positionne un drapeau si une trame a t correctement reue ou interromp la rception
et attend d'tre rinitialis par <timer1_ovf_isr> dans le cas contraire.

Le temps entre deux front est mesur par le timer1 du microcontrleur.
------------------------------------------------------------------------------------*/

#ifndef _RC5_INCLUDED_
#define _RC5_INCLUDED_


// Dfinition des vecteurs d'interruption utiliss avec le timer1
// --------------------------------------------------------------
#ifndef  TIM1_CAPT                 
#define  TIM1_CAPT	6
#endif

#ifndef  TIM1_OVF
#define  TIM1_OVF       9
#endif


// Etats du graphe "Acquisition d'une trame RC5" situ dans <timer1_capt_isr>
//---------------------------------------------------------------------------
#define ATTENTE_RC5				0
#define BIT_PRECEDENT_A_1_RC5 			1
#define ATTENTE_FRONT_DESCENDANT_RC5		2
#define BIT_PRECEDENT_A_0_RC5           	3  
#define ATTENTE_FRONT_MONTANT_RC5		4


// Front attendu et rglage de la frquence d'incrmentation du compteur
//-------------------------------------------------------------------------
#define ICP_ATTEND_FRONT_MONTANT	(TCCR1B=0x41) // ICP actif sur front montant, Clk Timer1 = 4MHz
#define ICP_ATTEND_FRONT_DESCENDANT	(TCCR1B=0x01) // ICP actif sur front descendant, Clk Timer1 = 4MHz
//Remarque: le dbruitage de ICP n'est pas utilis, la  rception s'est rvle moins bonne avec une tlcommande universelle                    


// Autorisation des interruptions
// ---------------------------------
#define INTERRUPT_DEBORD_PAS_CAPTURE		(TIMSK = 0x04)
#define INTERRUPT_DEBORD_et_CAPTURE             (TIMSK = 0x24)
                             

// Temps entre deux front du signal RC5
//--------------------------------------
#define duree	ICR1 			// A chaque front actif de ICP, le registre de capture du timer 1 (ICR1)
#define TMIN1	2400	// 0,6ms        // contient une image du temps coul depuis le dernier front actif
#define TMIN2	4800    // 1,2ms        // on a duree(s) = ICR1 * tclk (tclk = 250E-9s si FQuartz = 4MHz)
#define TMAX1	6400    // 1,6ms
#define TMAX2	8000    // 2ms


// Etat dde l'indicateur de trame
//-------------------------------
#define RECU			1  	// Trame reue
#define CONSOMME 		0       // Trame consomme



/*===============================
Prototype des sous-programme
=================================*/
#pragma used+

// -----------------------------------------------------------------------------
// Sous programme d'interruption dclench lors de la capture d'un front sur ICP
// -----------------------------------------------------------------------------
interrupt [TIM1_CAPT] void timer1_capt_isr(void);
/*
Rle: Dcoder et dsrialiser la trame RC5

Commentaires: Dcode les 14 bits de la trame RC5 transmis au format Manchester et les
dsrialise dans une variable globale <g_Trame>.
Si les 14 bits ont t correctement reus
	alors un drapeau <g_Caractere> est positionn avec la constante RECU
	sinon la reception est interrompue jusqu'au dbordement du timer1 (<voir timer1_ovf_isr>)
Lorsque le caractre reu est consomm par le programme principal celui-ci place la constante CONSOMME
dans la variable <g_Caractere>

Entre : vnement d'interruption (front actif sur ICP) 

Sortie : variable globale de type entier <g_Trame>  initialiser  0
	 variable globale de type char <g_Caractere>  initialiser  CONSOMME
*/


// -----------------------------------------------------------------------------
// Sous programme d'interruption dclench lors du dbordement du timer 1
// -----------------------------------------------------------------------------
interrupt [TIM1_OVF] void timer1_ovf_isr(void);
/*
Rle: Rinitialiser le sous-programme d'interruption <timer1_capt_isr> si le timer1 dborde

Commentaires: Le dbordement du registre de comptage du timer1 se produit aprs un temps
T_rearm = 65536 * Tclk avec Tclk = 250e-9s (si Fquartz = 4Mhz) soit T_rearm = 16,384ms
(si Fquartz = 4Mhz)
T_rearm permet de rinitialiser la reception aprs que celle-ci ait t annule par  
<timer1_capt_isr> lorsque les temps entre deux fronts ne sont pas conforment a ceux du
protocole RC5.
En l'absence de signal RC5, la rinitialisation s'effectue priodiquement toutes les 
16,384ms. 

Entre : vnement d'interruption (dbordement du timer1) 

Sortie : aucune
*/


#pragma used-     

#pragma library RC5.lib

#endif